/*  File DYNAMSYS.C Dynamical Systems: Henon and Ikeda Systems */

#ifndef DYNAMSYS_C
#define DYNAMSYS_C

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include <math.h>

#include "dyn.h"

dyn_setup_rec  g_dyn_setup;

const  descr_str dyn_descr[2] =
      {"Henon  ","Ikeda  "};

void dyn_f (float x_in,float y_in,float *x_out,float *y_out);
void display_dyn_param (dyn_setup_rec setup,short *line);
void initialize_dyn_setup (dyn_setup_rec *the_rec);

/*  Implementation  */

void dyn_f (float x_in,float y_in,float *x_out,float *y_out) {
   float u,cu,su;
   switch (g_dyn_setup.the_sys) {
   case Henon:
      *x_out = 1.0 + y_in - g_dyn_setup.hn.a * x_in * x_in;
      *y_out = g_dyn_setup.hn.b * x_in;
      break;
   case Ikeda:
      u = g_dyn_setup.ik.kappa -
		g_dyn_setup.ik.alpha/(1.0 + x_in*x_in + y_in*y_in);
      cu = cos(u);
      su = sin(u);
      *x_out = g_dyn_setup.ik.gamma + g_dyn_setup.ik.r*(x_in*cu - y_in*su);
      *y_out = g_dyn_setup.ik.r*(y_in*cu + x_in*su);
      break;
   }  /* end switch */
   return;
   }  /* end procedure */

void display_dyn_param (dyn_setup_rec setup,short *line) {
  (*line) += 2;
  gotoxy (1,*line);
  switch (setup.the_sys) {
  case Henon:
     cputs ("Henon System has been selected.");
     (*line) += 2;
     gotoxy (1,*line);
     cprintf ("\rParameter A: %6.4f",setup.hn.a);
     (*line)++;
     gotoxy (1,*line);
     cprintf ("\rParameter B: %6.4f",setup.hn.b);
     break;
  case Ikeda:
     cputs ("Ikeda System has been selected.");
     *line += 2;
     gotoxy (1,*line);
     cprintf ("\rParameter Alpha: %6.4f",setup.ik.alpha);
     (*line)++;
     gotoxy (1,*line);
     cprintf ("\rParameter Gamma: %6.4f",setup.ik.gamma);
     (*line)++;
     gotoxy (1,*line);
     cprintf ("\rParameter Kappa: %6.4f",setup.ik.kappa);
     (*line)++;
     gotoxy (1,*line);
     cprintf ("\rParameter R: %6.4f",setup.ik.r);
     break;
     }  /*  end switch  */
  *line += 2;
  return;
  } /* end proc */

void initialize_dyn_setup (dyn_setup_rec *setup) {
     setup->the_sys = Henon;
     strcpy(setup->sys_descr,dyn_descr[setup->the_sys]);
     /* Parameters for "classic" Henon strange attractor */
     setup->hn.a = 1.4;
     setup->hn.b = 0.3;
     /* Parameters for "classic" Ikeda strange attractor */
     setup->ik.alpha = 5.5;
     setup->ik.gamma = 0.85;
     setup->ik.kappa = 0.4;
     setup->ik.r = 0.9;
     setup->no_of_items = 400;
     setup->gr_rec.label_axes = 1;
     setup->gr_rec.x_axis_label = 'X';
     setup->gr_rec.y_axis_label = 'Y';
     setup->gr_rec.scale_window_from_data = 1;
     setup->gr_rec.max_mode = VGAHI;
     setup->gr_rec.x_min = -1.0;
     setup->gr_rec.x_max = 1.0;
     setup->gr_rec. y_min = -1.0;
     setup->gr_rec.y_max = 1.0;
     setup->gr_rec.back_color = BLUE;
     setup->gr_rec.text_color = YELLOW;
     setup->gr_rec.line_color = LIGHTCYAN;
     return;
   }  /* end proc */

#endif