/* File FFRUN.C Run a feedforward neural network  */
/*              and display results               */

#ifndef FFRUN_C
#define FFRUN_C

#include <conio.h>
#include <alloc.h>

#include "ut.h"
#include "trn.h"
#include "ff.h"

void ff_network (ff_setup_rec setup,training_rec tr_rec,weights_rec weights,
     get_data_proc get_network_input_output);
void run_ff_network (ff_setup_rec setup);

/* Implementation */

void ff_network (ff_setup_rec setup,training_rec tr_rec,weights_rec weights,
     get_data_proc get_network_input_output) {
     int i;
     f_vector d_out = NULL;
     network_rec network = {NULL,NULL,NULL};
     display_rec display = {0,0,0,0,0,0};
     float err;
     clrscr();
     allocate_network (setup,&network);
     d_out = allocate_f_vector (1,setup.no_of_outputs);
     display.iter = setup.start_iter;
     display.nbr_tr_items = tr_rec.no_of_training_items;
     setup_screen (setup,display);
     for (i = 1;i<= tr_rec.no_of_training_items;i++) {
	 get_network_input_output (setup,tr_rec,i,network.x_inp,d_out);
	 feedforward (network,weights,setup);
	 err = ff_error (d_out,network.y_out,setup); /* Normalized error */
	 if (err > setup.error_threshold)
	     display.wrong_count++;
	 else display.right_count++;
	 display_results (setup,display,d_out,network.y_out);
	 if (getch() == ESC_KEY) break;
	 } /* i */
     getch();
     clrscr();
     free_f_vector (d_out,1);
     free_network (&network);
     }  /* end proc */

void run_ff_network (ff_setup_rec setup) {
     weights_rec weights = {0,NULL,NULL};
     training_rec tr_rec;
     clrscr();
     textcolor (YELLOW);
     gotoxy (1,3);
     cputs ("Feedforward Neural Network: Run in forward mode");
     gotoxy (1,5);
     if (setup.files.in_wts_file_name[0] == '\0') {
	cputs ("\r\nInput weights file name is blank.\r\n"
	       "Press key to cancel: ");
	getch();
	return;
	}
     if (setup.files.training_file_name[0] == '\0') {
	cputs ("\r\nTraining file name is blank.\r\n"
		 "Press key to cancel: ");
	getch();
	return;
	}
     cprintf("Input weights file name: %s",setup.files.in_wts_file_name,'.');
     gotoxy (1,7);
     cprintf ("Training file name: %s.",setup.files.training_file_name);
     gotoxy (1,9);
     cputs ("Press any key to continue (esc to cancel): ");
     if (getch() == ESC_KEY) return;
     clrscr();
     if (!process_training_file (setup.files.training_file_name,
	setup.no_of_inputs,setup.no_of_outputs,&tr_rec))
	goto exit_proc;
     normalize_training_set (setup.no_of_inputs,setup.no_of_outputs,
	setup.norm_in_range,setup.norm_in_min,setup.norm_out_range,
	setup.norm_out_min,tr_rec);
     if (!read_weights (&weights,setup)) goto exit_proc;
     ff_network (setup,tr_rec,weights,standard_data_input);
exit_proc:
     free_training_pointers(&tr_rec);
     free_weights (&weights);
     return;
     }  /* end proc  */

#endif