/*  FFGENALG.C Genetic Algorithm "plug in" training unit */

#ifndef FFGENALG_C
#define FFGENALG_C

#include <conio.h>
#include <alloc.h>

#include "ut.h"
#include "ga.h"
#include "ffadv.h"

float ga_obj_fn (ui_vector ga_vect);

void genetic_alg_training (ff_setup_rec setup,
	   weights_rec weights,training_rec tr_rec,
	   get_data_proc get_network_input_output);

/*  Implementation  */

/* Used only in this file: */
int g_no_of_weights;
f_vector w_range,w_min;
float max_ff_obj_fn_value;

float ga_obj_fn (ui_vector ga_vect) {
   int i;
   d_vector weights;
   float ga_obj_fn_value;
   weights = allocate_d_vector (1,g_no_of_weights);
   for (i=0;i<g_no_of_weights;i++)
       weights[i+1] = (ga_vect[i]/g_ga_max[i])
	  *w_range[i] + w_min[i];
   ga_obj_fn_value = (MAX_GA_OBJ_FN_VALUE -
	ff_objective_fn (weights)/max_ff_obj_fn_value);
   free_d_vector (weights,1);
   return ga_obj_fn_value;
   }

void genetic_alg_training (ff_setup_rec setup,
	   weights_rec weights,training_rec tr_rec,
	   get_data_proc get_network_input_output) {
     /* 'get_network_input_output' is not used.  This paramater is */
     /*  needed to match format of training_proc.  */
     int i,j,k;
     float err;
     d_vector v_weights;
     g_tr_rec = tr_rec;
     g_no_of_weights = (setup.no_of_inputs + 1) * setup.no_of_h1_nodes +
	    setup.no_of_h1_nodes * setup.no_of_outputs;
     max_ff_obj_fn_value =
       compute_max_obj_fn_value (setup,tr_rec.no_of_training_items);
     ga_setup_rec.vect_len = g_no_of_weights;
     ga_setup_rec.chrom_len = ga_setup_rec.bit_len * ga_setup_rec.vect_len;
     ga_setup_rec.tolerance = setup.error_threshold;
     w_range = allocate_f_vector (0,g_no_of_weights-1);
     w_min = allocate_f_vector (0,g_no_of_weights-1);
     for (i=0;i<g_no_of_weights;i++) {
	  w_range[i] = 2 * setup.weight_range;
	  w_min[i] = -setup.weight_range;
	  }
     genetic_algorithm (&ga_setup_rec,&ga_rec,&ga_obj_fn);
     /* Get network weights from genetic output */
     v_weights = allocate_d_vector (1,g_no_of_weights);
     for (i=0;i<g_no_of_weights;i++)
	 v_weights[i+1] = ((ga_rec.opt_vector)[i]/g_ga_max[i])
	     *w_range[i] + w_min[i];
     weights_to_network_wts (setup,v_weights,weights);
     free_d_vector (v_weights,1);
     free_f_vector (w_range,0);
     free_f_vector (w_min,0);
     clrscr();
     gotoxy (1,4);
     if (write_weights (weights,setup.files.out_wts_file_name,setup)) {
	gotoxy (1,10);
	cprintf ("Weights saved to file: %s",setup.files.out_wts_file_name);
	gotoxy (1,12);
	cputs ("Press key to clear: ");
	getch();
	}
     textmode (LASTMODE);
     return;
     } /* end proc */

#endif