/*  File FFBRMRUN.C   Run Neural Network Broom Balancing System  */

#ifndef FFBRMRUN_C
#define FFBRMRUN_C

#include <stdio.h>
#include <conio.h>
#include <math.h>
#include "trn.h"
#include "ffbrm.h"

network_rec network = {0,NULL,NULL,NULL};
weights_rec weights = {0,NULL,NULL};
ff_setup_rec the_setup;
state_rec g_min_state,g_max_state;

void run_ff_controller(ff_setup_rec setup);

/* Implementation  */

float get_neural_network_force (state_rec state) {
   float out_angle;
   /*  The controller operates by "looking ahead", like an experienced player
       would. The trained network emulates the broom dynamics, so that the
       controller can ask:  "What will happen if I do nothing (i.e., apply
       0 force)?".  The network (not the actual broom dynamics) answers
       this question by supplying the broom angle that would result on
       the next iteration if 0 force were applied.  Once this is answered,
       the appropriate action can be taken. */
   get_network_input (the_setup,0.0,state,
	  g_min_state,network.x_inp);
   feedforward (network,weights,the_setup);
   /* Use 'angle' output to determine control force:  */
   out_angle = (network.y_out[1] - 0.5) * 2 * theta_factor;
   if (fabs (out_angle) < g_brm_setup.zero_theta)
      return  0.0;
   if (out_angle >= g_brm_setup.zero_theta)
      return g_brm_setup.system_force_incr;
   return -g_brm_setup.system_force_incr;
   }  /* end funct */

void run_ff_controller (ff_setup_rec setup) {
     training_rec tr_rec = {0,0,1.0,1.0,NULL,NULL,NULL,0,
	  NULL,NULL,NULL,NULL};
     the_setup = setup;
     clrscr();
     textcolor (YELLOW);
     gotoxy (1,3);
     cputs ("Neural Net Broom Balancing Recognition Program");
     gotoxy (1,5);
     if (setup.files.in_wts_file_name[0] == '\0') {
	cprintf ("\r\nNetwork Weights file name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     if (setup.files.training_file_name[0] == '\0') {
	cprintf ("\r\nTraining file name is blank.\r\n");
	cputs ("Press key to cancel: ");
	getch();
	return;
	}
     cprintf ("Input weights file name: %s.",setup.files.in_wts_file_name);
     gotoxy (1,7);
     cprintf ("Training file name: %s.",setup.files.training_file_name);
     gotoxy (1,11);
     cputs ("Press any key to continue (esc to cancel): ");
     if (getch() == ESC_KEY) return;
     clrscr();
     gotoxy (1,5);
     allocate_network (setup,&network);
     if (!load_max_min_data (setup,&g_min_state,&g_max_state,&tr_rec))
	goto exit_proc;
     cputs ("\r\nNetwork weights: ");
     if (!read_weights (&weights,setup)) goto exit_proc;
     control_system ("\"Feedforward Network Broom Balancer\"",g_brm_setup,
	&get_neural_network_force);

exit_proc:
     free_weights (&weights);
     free_training_pointers(&tr_rec);
     free_network (&network);
     textmode (LASTMODE);
     return;
     }  /* end proc */

#endif