/*  File FFINRUN.C Run a feedforward network for financial forecasting  */

#ifndef FFINRUN_C
#define FFINRUN_C

#include <stdio.h>
#include <conio.h>
#include <alloc.h>
#include <mem.h>
#include <math.h>
#include "ut.h"
#include "trn.h"
#include "ff.h"

void run_ffin_network (ff_setup_rec setup);

/* Implementation  */

char get_command (void) {
   const
      comx = 30,
      comy = 18;
   char command = ' ';
      gotoxy (comx,comy);
      cputs ("r: Run Network");
      gotoxy (comx,comy + 1);
      cputs ("n: Get New Input Data");
      gotoxy (comx,comy + 2);
      cputs ("p: Print All Items");
      gotoxy (comx,comy + 3);
      cputs ("f: Write Error Results to File");
      gotoxy (comx,comy + 4);
      cputs ("x: Exit");
      do {
	gotoxy (comx,comy + 6);
	cputs ("Response:  ");
	gotoxy (wherex() - 1,wherey());
	command = getch();
      } while ((command != 'r')&&(command != 'n')&&(command != 'p')
	    &&(command != 'f') && (command != 'x'));
      return command;
      } /*  function  */

int get_test_item (ff_setup_rec setup,training_rec tr_rec) {
    char str_item[5];
    int data_item = 0,i,line,line_skip;
    clrscr();
    do {
       gotoxy (1,5);
       cputs ("                                     ");
       gotoxy (1,5);
       cprintf ("Enter data item number (1 to %d): ",
		 tr_rec.no_of_training_items);
       gets (str_item);
       /* Scan for correct numeric input. */
       } while ((sscanf (str_item,"%d",&data_item) != 1) ||
	   ((data_item < 1) || (data_item > tr_rec.no_of_training_items)));
    if (setup.no_of_inputs < 10) line_skip = 2;
    else line_skip = 1;
    line = 12 - (setup.no_of_inputs / (3 - line_skip));
    if (line < 2) line = 2;
    gotoxy (1,line);
    for (i = 1;i<=setup.no_of_inputs;i++) {
	cprintf ("Input #%d: %7.2f",i,
	    tr_rec.training_array[data_item][i]);
	line += line_skip;
	gotoxy (1,line);
	}  /* i */
     return data_item;
     } /* proc */

void write_results_to_file (ff_setup_rec setup,
	  network_rec network,weights_rec weights) {
     FILE *outfile;
     int i,j;
     f_vector y = NULL,d_out = NULL,norm_d_out = NULL;
     training_rec norm_tr_rec = {0,0,1.0,1.0,NULL,NULL,NULL,0,
	  NULL,NULL,NULL,NULL};
     float err,norm_err,cum_err = 0.0,cum_norm_err = 0.0;
     clrscr();
     gotoxy (1,5);
     if (setup.files.data_file_name[0] == '\0') {
	cputs ("\r\nOutput Data file name is blank."
	  "\r\n\nPress key to cancel: ");
	getch();
	clrscr();
	return;
	}
     if (setup.files.training_file_name[0] == '\0') {
	cputs ("\r\nTraining file name is blank.\r\n\nPress key to cancel: ");
	getch();
	clrscr();
	return;
	}
     cprintf ("\rTraining data file: %s",setup.files.training_file_name);
     cprintf ("\r\n\nOutput data will be written to file: %s",
	setup.files.data_file_name);
     cputs ("\r\n\nPress key to start (esc to cancel): ");
     if (getch() == ESC_KEY) {
	clrscr();
	return;
	}
     if (!open_output_text_file (&outfile,setup.files.data_file_name)) {
	clrscr();
	return;
	}
     if (!process_training_file (setup.files.training_file_name,
	setup.no_of_inputs,setup.no_of_outputs,&norm_tr_rec)) {
	fclose (outfile);
	clrscr();
	return;
	};
     normalize_training_set (setup.no_of_inputs,setup.no_of_outputs,
	setup.norm_in_range,setup.norm_in_min,setup.norm_out_range,
	setup.norm_out_min,norm_tr_rec);

     y = allocate_f_vector (1,setup.no_of_outputs);
     d_out = allocate_f_vector (1,setup.no_of_outputs);
     norm_d_out = allocate_f_vector (1,setup.no_of_outputs);

     /* Write file header information:  */
     fprintf (outfile,"Training File: %s\n",setup.files.training_file_name);
     fprintf (outfile,"Weights File: %s\n",setup.files.in_wts_file_name);
     fprintf (outfile,"Network: %d  %d  %d\n",setup.no_of_inputs,
	setup.no_of_h1_nodes,setup.no_of_outputs);
     for (j = 1;j<=setup.no_of_outputs;j++)
	 fprintf (outfile,"    Net%2d ",j);
     fputs ("  ",outfile);
     for (j = 1;j<=setup.no_of_outputs;j++)
	 fprintf (outfile," Actual%2d ",j);
     fprintf (outfile,"      Error  Norm Error\n");
     for (i = 1;i<= norm_tr_rec.no_of_training_items;i++) {
	 /*  Move non-normalized training data to x and d_out  */
	 memmove (d_out + 1,norm_tr_rec.correct_answers[i] + 1,
		 (size_t)((setup.no_of_outputs)*sizeof(float)));
	 standard_data_input (setup,norm_tr_rec,i,network.x_inp,norm_d_out);
	 feedforward (network,weights,setup);
	 denormalize_output (network.y_out,y,setup.no_of_outputs,
	       setup.norm_out_range,setup.norm_out_min,norm_tr_rec);
	 for (j = 1;j<=setup.no_of_outputs;j++)
	     fprintf (outfile,"%9.2f ",y[j]);
	 fputs ("  ",outfile);
	 for (j = 1;j<=setup.no_of_outputs;j++)
	     fprintf (outfile,"%9.2f ",d_out[j]);
	 err = sqrt(ff_error (y,d_out,setup));
	 norm_err = sqrt(ff_error (network.y_out,norm_d_out,setup));
	 fprintf (outfile,"  %9.4f %9.4f\n",err,norm_err);
	 cum_err += err;
	 cum_norm_err += norm_err;
	 } /* i */
     /* Average errors: */
     cum_err /= norm_tr_rec.no_of_training_items;
     cum_norm_err /= norm_tr_rec.no_of_training_items;
     fprintf (outfile,"\n%9.4f  %9.4f\n",cum_err,cum_norm_err);
     fclose (outfile);
     free_f_vector (y,1);
     free_f_vector (d_out,1);
     free_f_vector (norm_d_out,1);
     free_training_pointers (&norm_tr_rec);
     clrscr();
     return;
     }  /* proc */

void print_results(ff_setup_rec setup,training_rec tr_rec,
	  network_rec network,weights_rec weights) {
     int i,j;
     f_vector x = NULL,y = NULL,d_out = NULL;
     float err;
     fprintf (stdprn,"Training File: %s\n",setup.files.training_file_name);
     fprintf (stdprn,"Test File: %s\n",setup.files.test_file_name);
     fprintf (stdprn,"Weights File: %s\n\n",setup.files.in_wts_file_name);
     fprintf (stdprn,"Number of hidden layer nodes: %d\n",setup.no_of_h1_nodes);
     fprintf (stdprn,"Item#  ");
     for (i = 1;i<= setup.no_of_inputs;i++)
	 fprintf (stdprn," Input #%1d ",i);
     fprintf (stdprn,"   ");
     for (i = 1;i<=setup.no_of_outputs;i++)
	 fprintf (stdprn," N Out #%1d ",i);
     fprintf (stdprn,"  ");
     for (i = 1;i<=setup.no_of_outputs;i++)
	 fprintf (stdprn," D Out #%1d ",i);
     fprintf (stdprn,"   Error\n");

     x = allocate_f_vector (1,setup.no_of_inputs);
     y = allocate_f_vector (1,setup.no_of_outputs);
     d_out = allocate_f_vector (1,setup.no_of_outputs);

     for (i = 1;i<= tr_rec.no_of_training_items;i++) {
	 /*  Move non-normalized training data to x and d_out  */
	 memmove (x + 1,tr_rec.training_array[i] + 1,
		 (size_t)((setup.no_of_inputs)*sizeof(float)));
	 memmove (d_out + 1,tr_rec.correct_answers[i] + 1,
		 (size_t)((setup.no_of_outputs)*sizeof(float)));
	 normalize_data (x,network.x_inp,setup.no_of_inputs,
	       tr_rec.max_in,tr_rec.min_in,setup.norm_in_range,
	       setup.norm_in_min);
	 feedforward (network,weights,setup);
	 denormalize_output (network.y_out,y,setup.no_of_outputs,
	       setup.norm_out_range,setup.norm_out_min,tr_rec);
	 fprintf (stdprn,"%3d.",i);
	 for (j = 1;j<=setup.no_of_inputs;j++)
	     fprintf (stdprn," %9.2f",x [j]);
	 fprintf (stdprn,"  %c",VERT_BAR);
	 for (j = 1;j<=setup.no_of_outputs;j++)
	     fprintf (stdprn," %9.2f",y[j]);
	 fprintf (stdprn," %c",VERT_BAR);
	 for (j = 1;j<=setup.no_of_outputs;j++)
	     fprintf (stdprn," %9.2f",d_out[j]);
	 err = sqrt(ff_error (y,d_out,setup));
	 fprintf (stdprn," %c    %5.3f\n",VERT_BAR,err);
	 } /* i */
     free_f_vector (x,1);
     free_f_vector (y,1);
     free_f_vector (d_out,1);
     return;
     }  /* proc */

void run_ffin_net (ff_setup_rec setup,training_rec tr_rec,
	  network_rec network,weights_rec weights,int data_item) {
     int i,j,k,line;
     f_vector x = NULL,y = NULL,d_out = NULL;
     float err;

     x = allocate_f_vector (1,setup.no_of_inputs);
     y = allocate_f_vector (1,setup.no_of_outputs);
     d_out = allocate_f_vector (1,setup.no_of_outputs);

     /*  Move non-normalized training data to x and d_out  */
     memmove (x + 1,tr_rec.training_array[data_item] + 1,
	 (size_t)((setup.no_of_inputs)*sizeof(float)));
     memmove (d_out + 1,tr_rec.correct_answers[data_item] + 1,
	 (size_t)((setup.no_of_outputs)*sizeof(float)));
     normalize_data (x,network.x_inp,setup.no_of_inputs,
	    tr_rec.max_in,tr_rec.min_in,setup.norm_in_range,
	    setup.norm_in_min);
     feedforward (network,weights,setup);
     denormalize_output (network.y_out,y,setup.no_of_outputs,
	    setup.norm_out_range,setup.norm_out_min,tr_rec);
     line = 12 - setup.no_of_outputs;
     if (line < 2) line = 2;
     gotoxy (50,line);
     cprintf ("Data Item %2d: ",data_item);
     line++;
     for (i = 1;i<=setup.no_of_outputs;i++) {
	 line++;
	 gotoxy (50,line);
	 cprintf ("Net Output[%d]:   %7.2f",i,y[i]);
	 }  /* i */
     line++;
     for (i = 1;i<=setup.no_of_outputs;i++) {
	 gotoxy (50,line);
	 cprintf ("Actual Value[%d]: %7.2f",i,d_out[i]);
	 }  /* i */
     free_f_vector (x,1);
     free_f_vector (y,1);
     free_f_vector (d_out,1);
     return;
     }  /* proc */

void run_ffin_network (ff_setup_rec setup) {
     weights_rec weights = {0,NULL,NULL};
     training_rec tr_rec = {0,0,1.0,1.0,NULL,NULL,NULL,0,
	  NULL,NULL,NULL,NULL};
     network_rec network = {0,NULL,NULL,NULL};
     int test_item_no = 1;
     char resp = ' ';
     clrscr();
     textcolor (YELLOW);
     gotoxy (1,3);
     cputs ("Neural Network Financial Forecasting: Run Network");
     gotoxy (1,5);
     if (setup.files.in_wts_file_name[0] == '\0') {
	cputs ("\r\nInput weights file name is blank."
	  "\r\n\nPress key to cancel: ");
	getch();
	return;
	}
     if (setup.files.test_file_name[0] == '\0') {
	cputs ("\r\nTest file name is blank.\r\n\nPress key to cancel: ");
	getch();
	return;
	}
     cprintf("Input weights file name: %s",setup.files.in_wts_file_name,'.');
     gotoxy (1,7);
     cprintf ("Test file name: %s.",setup.files.test_file_name);
     gotoxy (1,9);
     cputs ("Press any key to continue (esc to cancel): ");
     if (getch() == ESC_KEY) return;
     clrscr();
     if (!process_training_file (setup.files.test_file_name,
	setup.no_of_inputs,setup.no_of_outputs,&tr_rec))
	goto exit_proc;
     /* Don't want to normalize training set at this time  */
     if (!read_weights (&weights,setup)) goto exit_proc;
     allocate_network (setup,&network);
     test_item_no = 1;
     do {
	resp =  get_command ();
	switch (resp) {
	case 'r': run_ffin_net (setup,tr_rec,network,weights,test_item_no);
	     break;
	case 'n': test_item_no = get_test_item (setup,tr_rec);
	     break;
	case 'p': print_results (setup,tr_rec,network,weights);
	     break;
	case 'f': write_results_to_file (setup,network,weights);
	     break;
	     }  /* end switch */
	} while (resp != 'x');

exit_proc:
     free_network (&network);
     free_training_pointers(&tr_rec);
     free_weights (&weights);
     textmode (LASTMODE);
     return;
     }  /* end proc  */

#endif