/*  File TRNPROCS.C  Procedures for processing training file  */

/*  Note:  All arrays for input and output have starting index 1 (not 0) !*/

#ifndef TRNPROCS_C
#define TRNPROCS_C

#include <conio.h>
#include <string.h>
#include <alloc.h>
#include <math.h>

#include "uttypes.h"
#include "ut.h"
#include "trn.h"

training_rec g_tr_rec;

const float zero_check = 1e-8;

void free_training_pointers (training_rec *tr_rec);
short process_training_file (path_str the_file_name,int no_of_inputs,
   int no_of_outputs,training_rec *tr_rec);
void normalize_data (f_vector x,f_vector norm_x,int no_of_pts,
   f_vector x_max,f_vector x_min,float norm_range,float norm_min);
void normalize_training_set (int no_of_inputs,int no_of_outputs,
   float norm_in_range,float norm_in_min,float norm_out_range,
   float norm_out_min,training_rec tr_rec);
void denormalize_output (f_vector y,f_vector denorm_out,int no_of_outputs,
   float norm_out_range,float norm_out_min,training_rec tr_rec);

/* Implementation  */

void free_training_pointers (training_rec *tr_rec) {
     if (tr_rec->allocated) {
	free_matrix2d (tr_rec->training_array,1,1);
	free_matrix2d (tr_rec->correct_answers,1,1);
	free_matrix2d (tr_rec->norm_answers,1,1);
	}
     if (tr_rec->max_min_allocated) {
	free_f_vector (tr_rec->max_in,1);
	free_f_vector (tr_rec->min_in,1);
	free_f_vector (tr_rec->max_out,1);
	free_f_vector (tr_rec->min_out,1);
	}
     tr_rec->allocated = FALSE;
     tr_rec->max_min_allocated = FALSE;
     return;
     }  /* end proc  */

short check_network_size (int set_in,int set_out,FILE *infile,int line) {
   int  n_in,n_out;
   fscanf (infile,"%d  %d\n",&n_in,&n_out);
   gotoxy (1,line);
   cprintf ("No. of input nodes: %d",n_in);
   gotoxy (1,line + 2);
   cprintf ("No. of output nodes: %d",n_out);
   cprintf ("\r\nSetup inputs: %d  outputs:  %d  \r\n",
     set_in,set_out);
   if ((n_in != set_in) || (n_out != set_out)) {
       gotoxy (1,line + 6);
       cputs ("These numbers do not agree with setup.\r\n"
	      "Press any key to cancel: ");
       getch();
       return 0;
       }
   return 1;
   } /* end func */

short process_training_file (path_str the_file_name,int no_of_inputs,
   int no_of_outputs,training_rec *tr_rec) {
#define FIELD_LEN 255
   float value;  /* used with sscanf to avoid potential addressing problems */
   int i,j,k,rec_len;
   const display_line = 7;
   long file_size_in_bytes,remaining_bytes,current_pos;
   char char_buffer[FIELD_LEN],*the_str;
   FILE *the_file;
   tr_rec->allocated = FALSE;
   tr_rec->max_min_allocated = FALSE;
   tr_rec->no_of_training_items = 0;
   tr_rec->no_of_training_items_inv = 1;
   tr_rec->norm_error_scaling = 1;
   tr_rec->training_array = NULL;
   tr_rec->correct_answers = NULL;
   tr_rec->norm_answers = NULL;
   tr_rec->max_in = NULL;
   tr_rec->min_in = NULL;
   tr_rec->max_out = NULL;
   tr_rec->min_out = NULL;
   clrscr();
   gotoxy (1,5);
   file_size_in_bytes = open_input_text_file (&the_file,the_file_name);
   if (file_size_in_bytes == 0) return 0;   /* File name not found */
   if (!check_network_size (no_of_inputs,no_of_outputs,
	the_file,display_line)) return 0;
   cputs ("\r\nReading training/test items...\r\n");
   /* Allocate space max and min arrays */
   tr_rec->max_in = allocate_f_vector (1,no_of_inputs);
   tr_rec->min_in = allocate_f_vector (1,no_of_inputs);
   tr_rec->max_out = allocate_f_vector (1,no_of_outputs);
   tr_rec->min_out = allocate_f_vector (1,no_of_outputs);
   tr_rec->max_min_allocated = TRUE;
   /* Read past "minimum"  */
   the_str = fgets(char_buffer,FIELD_LEN,the_file);
   if (!the_str) return 0;
   /* Read min input data */
   for (i=1;i<=no_of_inputs;i++) {
       if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
       sscanf(char_buffer,"%f",&value);
       tr_rec->min_in[i] = value;
       }  /*  end i  */
   /* Read min output data */
   for (i=1;i<=no_of_outputs;i++) {
       if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
       sscanf(char_buffer,"%f",&value);
       tr_rec->min_out[i] = value;
       }
   /* Read past "maximum"  */
   if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
   /* Read max input data */
   for (i=1;i<=no_of_inputs;i++) {
       if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
       sscanf(char_buffer,"%f",&value);
       tr_rec->max_in[i] = value;
       }
   /* Read max output data */
   for (i=1;i<=no_of_outputs;i++) {
       if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
       sscanf(char_buffer,"%f",&value);
       tr_rec->max_out[i] = value;
       }
   /* Read past "Training Data"  */
   if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
   /* Determine remaining size of file so we can calculate number of */
   /*    training items (for memory allocation). */
   current_pos = ftell(the_file);
   remaining_bytes = file_size_in_bytes - current_pos;
   if (remaining_bytes > 0) {
   /* First, we need the length of one record */
   /* When creating training files, be sure that all data records  */
   /* in the file have the same length, otherwise this read method */
   /* won't work. */
   if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
   rec_len = ftell(the_file) - current_pos;
   /* Restore file to start of data */
   fseek (the_file,current_pos,SEEK_SET);
   tr_rec->no_of_training_items =
      remaining_bytes/((no_of_inputs + no_of_outputs) * rec_len);
   tr_rec->no_of_training_items_inv =
      1.0/(tr_rec->no_of_training_items);
   /* Now do dynamic allocation for training items */
   tr_rec->training_array =
      matrix2d (1,tr_rec->no_of_training_items,1,no_of_inputs);
   tr_rec->correct_answers =
      matrix2d (1,tr_rec->no_of_training_items,1,no_of_outputs);
   tr_rec->norm_answers =
      matrix2d (1,tr_rec->no_of_training_items,1,no_of_outputs);
   tr_rec->allocated = TRUE;
   for (i=1;i<=tr_rec->no_of_training_items;i++) {
       for (j=1;j<=no_of_inputs;j++) {
	   if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
	   sscanf(char_buffer," %f",&value);
	   tr_rec->training_array[i][j] = value;
	   }  /* end j  */
       for (j=1;j<=no_of_outputs;j++) {
	   if (!fgets(char_buffer,FIELD_LEN,the_file)) return 0;
	   sscanf(char_buffer," %f",&value);
	   tr_rec->correct_answers[i][j] = value;
	   }  /* end j  */
       }  /*  end i  */
   } /* if  */

   fclose (the_file);
   gotoxy (1,wherey() + 2);
   cprintf ("Done.");
   gotoxy (1,wherey() + 2);
   cprintf ("%d training items processed.",tr_rec->no_of_training_items);
   gotoxy (1,wherey() + 2);
   cprintf ("Press key to continue (esc to cancel): ");
   if (getch() == ESC_KEY) return 0;
   return 1;
   }  /* end func  */

void normalize_data (f_vector x,f_vector norm_x,int no_of_pts,
   f_vector x_max,f_vector x_min,float norm_range,float norm_min) {
   /* Same normalization routine used for both inputs and outputs */
   /* Assumes vector index range is 1..no_of_pts for all vectors  */
   int i;
   for (i=1;i<=no_of_pts;i++) {
       if (fabs (x_max[i] - x_min[i]) > zero_check) {
	  if (x[i] >= x_max[i]) /* Greater than max, set equal to max  */
	     norm_x[i] = norm_range + norm_min;
	  if (x[i] <= x_min[i]) /* Less than min, set equal to min  */
	     norm_x[i] = norm_min;
	  if ((x[i] > x_min[i]) && (x[i] < x_max[i])) {
	     norm_x[i] = (x[i] - x_min[i])/
		   (x_max[i] - x_min[i]);
	     norm_x[i] = norm_range * norm_x[i] + norm_min;
	     }
	  }
       else
       /* Max = min, so data is constant: */
       norm_x[i] = 0.5 * norm_range + norm_min;
       }  /* end i */
      return;
      }  /* end func */

void normalize_training_set (int no_of_inputs,int no_of_outputs,
   float norm_in_range,float norm_in_min,float norm_out_range,
   float norm_out_min,training_rec tr_rec) {
   int i,j;
   /* Normalize training set input and output */
   for (i=1;i<=tr_rec.no_of_training_items;i++) {
       normalize_data (tr_rec.training_array[i],
	      tr_rec.training_array[i],
	      no_of_inputs,tr_rec.max_in,tr_rec.min_in,norm_in_range,
	      norm_in_min);
       normalize_data (tr_rec.correct_answers[i],
	      tr_rec.norm_answers[i],
	      no_of_outputs,tr_rec.max_out,tr_rec.min_out,norm_out_range,
	      norm_out_min);
       }  /*  end i */
   if (fabs(norm_out_range) > 1e-6)
      tr_rec.norm_error_scaling = tr_rec.no_of_training_items_inv/norm_out_range;
   return;
   }  /*  end proc  */

void denormalize_output (f_vector y,f_vector denorm_out,int no_of_outputs,
   float norm_out_range,float norm_out_min,training_rec tr_rec) {
   /* Translate normalized output values to regular output value range. */
   /* Assumes index range for vectors is 1..no_of_pts.  */
   int i;
   for (i=1;i<=no_of_outputs;i++) {
       denorm_out[i] = (y[i] - norm_out_min)/norm_out_range;
       denorm_out[i] = denorm_out[i] *
	  (tr_rec.max_out[i] - tr_rec.min_out[i]) +
	      tr_rec.min_out[i];
       }  /* i */
   return;
   }  /* end procedure */

#endif